"""Top-level package for drf-restricted-fields."""

__author__ = """Tonye Jack"""
__email__ = 'jtonye@ymail.com'
__version__ = '0.0.1'

from .mixins import RestrictedFieldsSerializerMixin

__all__ = ["RestrictedFieldsSerializerMixin"]



from collections import OrderedDict

from rest_framework.fields import SkipField
from rest_framework.relations import PKOnlyObject


class RestrictedFieldsSerializerMixin(object):
    """API Serializer mixin

    This provides support for restricting serialized data to only a subset of fields using the
    ``only`` and ``defer`` query parameters.

    :param only: Restricted to only a subset of fields (Include these fields).
    :type only: list
    :param defer: Defer the listed fields (Exclude these fields).
    :type defer: list


    **Example:**

    Serialize only the `id` and `name` fields.

    .. code-block:: console

        GET https://.../api/users/?only=id&only=name

    .. code-block:: console

        {
            "count":1,
            "next": "http://127.0.0.1:8000/api/users/?only=id&only=name&page=1",
            "previous":null,
            "results":[
                {
                    "id":1,
                    "name": "Test"
                },
                ...
            ],
        }
    """
    RESTRICTED_FIELDS_PARAM = 'only'
    DEFERRED_FIELDS_PARAM = 'defer'

    def to_representation(self, instance):
        """
        Convert Model Object instance -> Dict of primitive datatypes.

        :param instance: The django model instance.
        :type instance: django.db.models.Model
        :return: Dictionary of fields and corresponding value.
        """

        request = self.context['request']
        ret = OrderedDict()
        restricted_fields = request.query_params.getlist(self.RESTRICTED_FIELDS_PARAM)
        deferred_fields = request.query_params.getlist(self.DEFERRED_FIELDS_PARAM)
        fields = self._readable_fields

        if restricted_fields:
            fields = [f for f in fields if f.field_name in restricted_fields]

        if deferred_fields:
            fields = [f for f in fields if f.field_name not in deferred_fields]

        for field in fields:
            try:
                attribute = field.get_attribute(instance)
            except SkipField:
                continue

            # We skip `to_representation` for `None` values so that fields do
            # not have to explicitly deal with that case.
            #
            # For related fields with `use_pk_only_optimization` we need to
            # resolve the pk value.
            check_for_none = attribute.pk if isinstance(attribute, PKOnlyObject) else attribute
            if check_for_none is None:
                ret[field.field_name] = None
            else:
                ret[field.field_name] = field.to_representation(attribute)

        return ret



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

deploy_requires = [
    "bump2version",
    "readme_renderer[md]",
    "changes",
    "git-changelog",
    "twine",
    "wheel",
]

docs_requires = [
    "watchdog[watchmedo]",
    "Sphinx",
]

install_requires = [
    'django',
    'djangorestframework',
]

setup_requires = ['pytest-runner']

test_requires = [
    'pytest>=3',
    "tox",
    "tox-gh-actions",
    "coverage",
]

lint_requires = [
    "flake8",
    "yamllint",
    "isort",
    "black",
]

extras_require = {
    "development": [
        install_requires,
        deploy_requires,
        test_requires,
        lint_requires,
    ],
    'development:python_version >= "3.6"': ["black"],
    "test": test_requires,
    "lint": lint_requires,
    'lint:python_version >= "3.6"': ["black"],
    "deploy": deploy_requires,
    "docs": docs_requires,
}

setup(
    author="Tonye Jack",
    author_email='jtonye@ymail.com',
    python_requires='>=2.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    description="Restrict fields returned by DRF serializers",
    install_requires=install_requires,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/x-rst",
    include_package_data=False,
    keywords=['drf_restricted_fields', 'restricted_fields'],
    name='drf_restricted_fields',
    packages=find_packages(include=['restricted_fields', 'restricted_fields.*']),
    setup_requires=setup_requires,
    test_suite='tests',
    tests_require=test_requires,
    extras_require=extras_require,
    url='https://github.com/tj-django/drf-restricted-fields',
    version='0.0.1',
    zip_safe=False,
)
